@@ -55,25 +55,25 @@ impl Parser {
         }
     }
 
-    pub fn tables(&self) -> Vec<&str> {
+    pub fn tables(&self) -> Vec<Vec<&str>> {
         from_query(&self.query)
     }
 }
 
-fn from_object_name(obj: &ObjectName) -> &str {
-    &obj.0.iter().last().unwrap().value
+fn from_object_name(obj: &ObjectName) -> Vec<&str> {
+    obj.0.iter().map(|id| id.value.as_str()).collect()
 }
 
-fn from_table_factor(table_factor: &TableFactor) -> Vec<&str> {
+fn from_table_factor(table_factor: &TableFactor) -> Vec<Vec<&str>> {
     match table_factor {
         TableFactor::Table { name, .. } => vec![from_object_name(name)],
         TableFactor::Derived { subquery, .. } => from_query(subquery),
         TableFactor::NestedJoin(table_with_joins) => from_table_with_joins(table_with_joins),
-        _ => todo!(),
+        _ => vec![],
     }
 }
 
-fn from_table_with_joins(table_with_joins: &TableWithJoins) -> Vec<&str> {
+fn from_table_with_joins(table_with_joins: &TableWithJoins) -> Vec<Vec<&str>> {
     let mut froms = from_table_factor(&table_with_joins.relation);
     let join_froms: Vec<_> = table_with_joins
         .joins
@@ -84,7 +84,7 @@ fn from_table_with_joins(table_with_joins: &TableWithJoins) -> Vec<&str> {
     froms
 }
 
-fn from_set_expr(set_expr: &SetExpr) -> Vec<&str> {
+fn from_set_expr(set_expr: &SetExpr) -> Vec<Vec<&str>> {
     match set_expr {
         SetExpr::Select(select) => from_select(select),
         SetExpr::Query(query) => from_query(query),
@@ -98,7 +98,7 @@ fn from_set_expr(set_expr: &SetExpr) -> Vec<&str> {
     }
 }
 
-fn from_expr(expr: &Expr) -> Vec<&str> {
+fn from_expr(expr: &Expr) -> Vec<Vec<&str>> {
     match expr {
         Expr::IsNull(expr) => from_expr(expr),
         Expr::IsNotNull(expr) => from_expr(expr),
@@ -231,7 +231,7 @@ fn from_expr(expr: &Expr) -> Vec<&str> {
     }
 }
 
-fn from_select_item(select_item: &SelectItem) -> Vec<&str> {
+fn from_select_item(select_item: &SelectItem) -> Vec<Vec<&str>> {
     match select_item {
         SelectItem::UnnamedExpr(expr) => from_expr(expr),
         SelectItem::ExprWithAlias { expr, .. } => from_expr(expr),
@@ -239,7 +239,7 @@ fn from_select_item(select_item: &SelectItem) -> Vec<&str> {
     }
 }
 
-fn from_select(select: &Select) -> Vec<&str> {
+fn from_select(select: &Select) -> Vec<Vec<&str>> {
     let mut projection_res: Vec<_> = select
         .projection
         .iter()
@@ -287,7 +287,7 @@ fn from_select(select: &Select) -> Vec<&str> {
     projection_res
 }
 
-fn from_query(query: &Query) -> Vec<&str> {
+fn from_query(query: &Query) -> Vec<Vec<&str>> {
     let mut froms = from_set_expr(&query.body);
     let order_by_res = query
         .order_by
@@ -310,7 +310,7 @@ fn from_query(query: &Query) -> Vec<&str> {
     }
     if let Some(with) = &query.with {
         let aliases = alias_with(with);
-        froms.retain(|from| !aliases.contains(from));
+        froms.retain(|from| !(from.len() == 1 && aliases.contains(&from[0])));
         let with_froms: Vec<_> = with
             .cte_tables
             .iter()
@@ -341,7 +341,7 @@ mod tests {
             Parser::try_new(SqlDialect::PostgreSql, query)
                 .unwrap()
                 .tables(),
-            vec!["cool_table"]
+            vec![vec!["cool_table"]]
         );
     }
 
@@ -350,7 +350,7 @@ mod tests {
         let query = "select * from cool_table union all select * from other_table";
         let parser = Parser::try_new(SqlDialect::PostgreSql, query).unwrap();
         let tables = parser.tables();
-        assert_eq!(tables, vec!["cool_table", "other_table"]);
+        assert_eq!(tables, vec![vec!["cool_table"], vec!["other_table"]]);
     }
 
     #[test]
@@ -364,7 +364,7 @@ mod tests {
         ";
         let parser = Parser::try_new(SqlDialect::PostgreSql, query).unwrap();
         let tables = parser.tables();
-        assert_eq!(tables, vec!["table_a", "table_b"]);
+        assert_eq!(tables, vec![vec!["table_a"], vec!["table_b"]]);
     }
 
     #[test]
@@ -377,7 +377,7 @@ mod tests {
         ";
         let parser = Parser::try_new(SqlDialect::PostgreSql, query).unwrap();
         let tables = parser.tables();
-        assert_eq!(tables, vec!["cool_table"])
+        assert_eq!(tables, vec![vec!["cool_table"]])
     }
 
     #[test]
@@ -394,7 +394,7 @@ mod tests {
         ";
         let parser = Parser::try_new(SqlDialect::PostgreSql, query).unwrap();
         let tables = parser.tables();
-        assert_eq!(tables, vec!["actual_table", "unique_table"]);
+        assert_eq!(tables, vec![vec!["actual_table"], vec!["unique_table"]]);
     }
 
     #[test]
@@ -406,7 +406,7 @@ mod tests {
         ";
         let parser = Parser::try_new(SqlDialect::PostgreSql, query).unwrap();
         let tables = parser.tables();
-        assert_eq!(tables, vec!["cool_table"])
+        assert_eq!(tables, vec![vec!["cool_table"]])
     }
 
     #[test]
@@ -427,7 +427,10 @@ mod tests {
         ";
         let parser = Parser::try_new(SqlDialect::PostgreSql, query).unwrap();
         let tables = parser.tables();
-        assert_eq!(tables, vec!["active_customers", "customers", "sales"]);
+        assert_eq!(
+            tables,
+            vec![vec!["active_customers"], vec!["customers"], vec!["sales"]]
+        );
     }
 
     #[test]
@@ -470,12 +473,12 @@ mod tests {
         assert_eq!(
             tables,
             vec![
-                "my_amount_table",
-                "my_lookup_table",
-                "my_main_table",
-                "my_other_table",
-                "my_side_table",
-                "my_table"
+                vec!["my_amount_table"],
+                vec!["my_lookup_table"],
+                vec!["my_main_table"],
+                vec!["my_other_table"],
+                vec!["my_side_table"],
+                vec!["my_table"]
             ]
         );
     }
